package sync

import (
	"sync"
	"testing"
)

// WaitGroup
func TestWaitGroup(t *testing.T) {

	var wg sync.WaitGroup

	for i := 0; i < 10; i++ {
		wg.Add(1)
		go func(num int) {
			defer wg.Done()
			println("num:", num)
		}(i)
	}
	wg.Wait()
}


// 不加锁
type Counter struct {
	count int
}

func NewCounter() *Counter {
	return &Counter{}
}

func (c *Counter) Increment() {
	c.count++
}

func (c *Counter) Get() int {
	return c.count
}

// 加锁 sync.Mutex
type MutexCounter struct {
	mu    sync.Mutex
	count int
}

func NewMutexCounter() *MutexCounter {
	return &MutexCounter{}
}

func (c *MutexCounter) Increment() {
	c.mu.Lock()
	defer c.mu.Unlock()
	c.count++
}

func (c *MutexCounter) Get() int {
	c.mu.Lock()
	defer c.mu.Unlock()
	return c.count
}

// 不加锁情况
func TestCounter(t *testing.T) {

	counter := NewCounter()
	var wg sync.WaitGroup

	for i := 0; i < 1000; i++ {
		wg.Add(1)
		go func() {
			defer wg.Done()
			counter.Increment()
		}()
	}

	wg.Wait()
	println("count: ", counter.Get()) // 不确定

}

// 加锁情况
func TestMutex(t *testing.T) {
	counter := NewMutexCounter()
	var wg sync.WaitGroup

	for i := 0; i < 1000; i++ {
		wg.Add(1)
		go func() {
			defer wg.Done()
			counter.Increment()
		}()
	}

	wg.Wait()
	println("count: ", counter.Get()) // 1000

}

type RWMutexCounter struct {
	mu    sync.RWMutex
	count int
}

func NewRWMutexCounter() *RWMutexCounter {
	return &RWMutexCounter{}
}

func (c *RWMutexCounter) Increment() {
	c.mu.Lock() // “写”锁
	defer c.mu.Unlock()
	c.count++
}

func (c *RWMutexCounter) Get() int {
	c.mu.RLock() // “读”锁
	defer c.mu.RUnlock()
	return c.count
}

// 读写锁
func TestRWMutex(t *testing.T) {
	counter := NewRWMutexCounter()
	var wg sync.WaitGroup

	for i := 0; i < 1000; i++ {
		wg.Add(1)
		go func() {
			defer wg.Done()
			counter.Increment()
		}()
	}

	wg.Wait()
	println("count: ", counter.Get()) // 1000
}

// TODO Once
// TODO Map
// TODO Pool
