package net

import (
	"fmt"
	"io"
	"net/http"
	"net/http/cookiejar"
	"net/url"
	"strings"
	"testing"

	"golang.org/x/net/publicsuffix"
)

// 以爬虫的视角解析 net/http

var tclient = &http.Client{}

func PrintResp(resp *http.Response) {
	res, err := io.ReadAll(resp.Body)
	if err != nil {
		fmt.Println(err)
	}
	fmt.Printf("%v\n", string(res))
}

func TestReqQickStart(t *testing.T) {
	resp, err := http.Get("https://httpbin.org/get")
	if err != nil {
		fmt.Println(err)
	}

	defer resp.Body.Close()

	body, err := io.ReadAll(resp.Body)
	if err != nil {
		fmt.Println(err)
	}

	fmt.Println(string(body))
}

// params
func TestQuery(t *testing.T) {
	req, err := http.NewRequest(http.MethodGet, "https://httpbin.org/get?q=TestQuery", nil)
	if err != nil {
		fmt.Println(err)
	}

	resp, err := tclient.Do(req)
	if err != nil {
		fmt.Println(err)
	}
	PrintResp(resp)
}

// headers
func TestHeaders(t *testing.T) {
	req, err := http.NewRequest(http.MethodGet, "https://httpbin.org/get?q=TestHeaders", nil)
	if err != nil {
		fmt.Println(err)
	}
	// 添加 Header
	req.Header.Add("TestAddHeader", "TestHeaders")
	// 修改 Header
	req.Header.Set("TestAddHeader", "TestHeaders Set")
	resp, err := tclient.Do(req)
	if err != nil {
		fmt.Println(err)
	}
	defer resp.Body.Close()

	PrintResp(resp)
}

// Cookies
func TestCookies(t *testing.T) {
	req, err := http.NewRequest(http.MethodGet, "https://httpbin.org/cookies", nil)
	if err != nil {
		fmt.Println(err)
	}

	// header cookie
	req.Header.Add("cookie", "TestHeaderCookie=Header.Add()")

	// AddCookie
	req.AddCookie(
		&http.Cookie{
			Name:  "TestAddCookie",
			Value: "req.AddCookie()",
		})

	for i := 0; i < 3; i++ {
		resp, err := tclient.Do(req)
		if err != nil {
			fmt.Println(err)
		}
		defer resp.Body.Close()

		PrintResp(resp)
	}
	queryResp(tclient)

}

func TestCookieJar(t *testing.T) {
	// session
	jar, err := cookiejar.New(&cookiejar.Options{PublicSuffixList: publicsuffix.List})
	tclient = &http.Client{
		Jar: jar,
	}

	var cookies []*http.Cookie
	jarCookie := &http.Cookie{
		Name:  "Testcookiejar",
		Value: "cookiejar.New(nil)",
	}
	cookies = append(cookies, jarCookie)

	cookieURL, _ := url.Parse("https://httpbin.org/cookies/set?TestURLCookie=FirstTestCookie")

	tclient.Jar.SetCookies(cookieURL, cookies)

	if err != nil {
		fmt.Println(err)
	}

	for i := 0; i < 3; i++ {
		req, err := http.NewRequest(http.MethodGet, "https://httpbin.org/cookies/set?TestURLCookie=FirstTestCookie", nil)
		if err != nil {
			fmt.Println(err)
		}
		resp, err := tclient.Do(req)

		if err != nil {
			fmt.Println(err)
		}
		defer resp.Body.Close()
		PrintResp(resp)
	}

	queryResp(tclient)
}

func queryResp(c *http.Client) {
	req, err := http.NewRequest(http.MethodGet, "https://httpbin.org/get?q=queryResp", nil)
	if err != nil {
		fmt.Println(err)
	}

	resp, err := c.Do(req)
	if err != nil {
		fmt.Println(err)
	}
	defer resp.Body.Close()

	fmt.Println("---------query resopnse----------------")
	PrintResp(resp)
}

func TestSessionExample(t *testing.T) {
	jar, err := cookiejar.New(nil)
	tclient = &http.Client{
		Jar: jar,
	}

	if err != nil {
		fmt.Println(err)
	}

	payload := strings.NewReader("username=123&password=123")

	req, err := http.NewRequest(http.MethodPost, "https://login2.scrape.center/login", payload)
	req.Header.Add("Host", "login2.scrape.center")
	req.Header.Add("Cache-Control", "max-age=0")
	req.Header.Add("sec-ch-ua", "\"Not A(Brand\";v=\"99\", \"Google Chrome\";v=\"121\", \"Chromium\";v=\"121\"")
	req.Header.Add("sec-ch-ua-mobile", "?0")
	req.Header.Add("sec-ch-ua-platform", "\"Windows\"")
	req.Header.Add("Upgrade-Insecure-Requests", "1")
	req.Header.Add("Origin", "https://login2.scrape.center")
	req.Header.Add("User-Agent", "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/121.0.0.0 Safari/537.36")
	req.Header.Add("Accept", "text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.7")
	req.Header.Add("Sec-Fetch-Site", "same-origin")
	req.Header.Add("Sec-Fetch-Mode", "navigate")
	req.Header.Add("Sec-Fetch-User", "?1")
	req.Header.Add("Sec-Fetch-Dest", "document")
	req.Header.Add("Referer", "https://login2.scrape.center/login")
	req.Header.Add("Accept-Language", "en,zh;q=0.9,zh-CN;q=0.8,zh-TW;q=0.7")
	req.Header.Add("Content-Type", "application/x-www-form-urlencoded")
	if err != nil {
		fmt.Println(err)
	}
	resp, err := tclient.Do(req)

	if err != nil {
		fmt.Println(err)
	}
	defer resp.Body.Close()
	cookieURL, _ := url.Parse("https://login2.scrape.center/login")
	fmt.Println(tclient.Jar.Cookies(cookieURL)) // session
	pageResp(tclient)

}

func pageResp(c *http.Client) {
	url := "https://login2.scrape.center/page/2"
	method := "GET"

	req, err := http.NewRequest(method, url, nil)

	if err != nil {
		fmt.Println(err)
		return
	}

	res, err := c.Do(req)
	if err != nil {
		fmt.Println(err)
		return
	}
	defer res.Body.Close()

	PrintResp(res)
	// fmt.Println(req.Cookies())
}
