package string

import (
	"fmt"
	"strings"
	"testing"
	"unicode"
	"unsafe"
)

// 复制字符串
func TestClone(t *testing.T) {
	s := "abc"
	// strings.Clone 浅拷贝
	/*
		func Clone(s string) string {
			if len(s) == 0 {
				return ""
			}
			b := make([]byte, len(s)) // 分配新的内存空间
			copy(b, s)
			return unsafe.String(&b[0], len(b))
		}

	*/

	clone := strings.Clone(s)

	fmt.Println(s == clone)
	fmt.Println(unsafe.StringData(s) == unsafe.StringData(clone))
}

// 比较两个字符串的字典排序值 (以字母顺序为基础的排序方式)
func TestCompare(t *testing.T) {
	/*
		func Compare(a, b string) int {
			if a == b {
				return 0
			}
			if a < b {
				return -1
			}
			return +1
		}

	*/
	fmt.Println(strings.Compare("a", "b"))
	fmt.Println(strings.Compare("a", "a"))
	fmt.Println(strings.Compare("b", "a"))
}

// 子串是否在主串内
func TestContains(t *testing.T) {
	// Contains
	fmt.Println("Contains:", strings.Contains("seafood", "foo"))
	fmt.Println("Contains:", strings.Contains("seafood", "bar"))
	fmt.Println("Contains:", strings.Contains("seafood", ""))
	fmt.Println("Contains:", strings.Contains("", ""))

	// ContainsAny - 子串任意一个字符在主串内
	fmt.Println("ContainsAny:", strings.ContainsAny("team", "i"))
	fmt.Println("ContainsAny:", strings.ContainsAny("fail", "ui"))
	fmt.Println("ContainsAny:", strings.ContainsAny("ure", "ui"))
	fmt.Println("ContainsAny:", strings.ContainsAny("failure", "ui"))
	fmt.Println("ContainsAny:", strings.ContainsAny("foo", ""))
	fmt.Println("ContainsAny:", strings.ContainsAny("", ""))

	// ContainsFunc || ContainsRune
	// func ContainsFunc(s string, f func(rune) bool) bool
	condition := func(r rune) bool {
		return r == 'o'
	}
	fmt.Println("ContainsFunc:", strings.ContainsFunc("o", condition))
}

// 子串在主串中的数量
func TestCount(t *testing.T) {

	fmt.Println(strings.Count("cheese", "es")) // 3

	// If substr is an empty string, Count returns 1 + the number of Unicode code points in s.
	fmt.Println(strings.Count("five", "")) // 5 - before & after each rune

}

// 按照给定字符切断主串
func TestCut(t *testing.T) {
	show := func(s, sep string) {
		before, after, found := strings.Cut(s, sep)
		fmt.Printf("Cut(%q, %q) = %q, %q, %v\n", s, sep, before, after, found)
	}
	show("Gopher", "Go")
	show("Gopher", "ph")
	show("Gopher", "er")
	show("Gopher", "Badger")
}

// 以空格分割字符串，如果字符串为空则返回空切片
func TestFields(t *testing.T) {
	// Fields
	fmt.Printf("Fields are: %q", strings.Fields("  foo bar  baz   "))

	// FieldsFunc
	f := func(c rune) bool {
		// 是否是字母和数字
		return !unicode.IsLetter(c) && !unicode.IsNumber(c)
	}
	fmt.Printf("Fields are: %q", strings.FieldsFunc("  foo1;bar2,baz3...", f))
}

// 检查字符串是否以子串开头或结尾
func TestHasPrefixAndSuffix(t *testing.T) {
	fmt.Println("HasPrefix:", strings.HasPrefix("Gopher", "Go"))
	fmt.Println("HasPrefix:", strings.HasPrefix("Gopher", "C"))
	fmt.Println("HasPrefix:", strings.HasPrefix("Gopher", ""))
	fmt.Println("HasSuffix:", strings.HasSuffix("Amigo", "go"))
	fmt.Println("HasSuffix:", strings.HasSuffix("Amigo", "O"))
	fmt.Println("HasSuffix:", strings.HasSuffix("Amigo", "Ami"))
	fmt.Println("HasSuffix:", strings.HasSuffix("Amigo", ""))
}

// `rune` 类型的字符文字
func TestRune(t *testing.T) {
	character := 'A'
	// 打印 rune 类型的值
	fmt.Printf("字符 %c 的 Unicode 码点是 %d\n", character, character)

	str := "A Hello"
	for _, v := range str {
		fmt.Printf("%q %d \n", v, v)
	}
}

// Repeat 字符串
func TestRepeat(t *testing.T) {
	fmt.Println("ba" + strings.Repeat("na", 2))
}

// Replace 替换字符串
func TestReplace(t *testing.T) {
	// Replace
	fmt.Println(strings.Replace("oink oink oink", "k", "ky", 2))
	fmt.Println(strings.Replace("oink oink oink", "oink", "moo", -1))

	// ReplaceAll
	fmt.Println(strings.ReplaceAll("oink oink oink", "oink", "moo"))
}

// Split 分割字符串
func TestSplit(t *testing.T) {
	fmt.Printf("%q\n", strings.Split("a,b,c", ","))
	fmt.Printf("%q\n", strings.Split("a man a plan a canal panama", "a "))
	fmt.Printf("%q\n", strings.Split(" xyz ", ""))
	fmt.Printf("%q\n", strings.Split("", "Bernardo O'Higgins"))
}

// Join 拼接字符串
func TestJoin(t *testing.T) {
	s := []string{"foo", "bar", "baz"}
	fmt.Println(strings.Join(s, ", "))
}

// 大小写相等
func TestEqualFold(t *testing.T) {
	fmt.Println(strings.EqualFold("Go", "go"))
	fmt.Println(strings.EqualFold("AB", "ab")) // true because comparison uses simple case-folding
	fmt.Println(strings.EqualFold("ß", "ss"))  // false because comparison does not use full case-folding
}

func TestToAny(t *testing.T) {
	// ToLower and ToUpper
	fmt.Println(strings.ToLower("Gopher"))
	fmt.Println(strings.ToLowerSpecial(unicode.TurkishCase, "Önnek İş"))
	fmt.Println(strings.ToUpper("Gopher"))
	fmt.Println(strings.ToUpperSpecial(unicode.TurkishCase, "örnek iş"))

	// ToTitle
	fmt.Println(strings.ToTitle("her royal highness"))
	fmt.Println(strings.ToTitle("loud noises"))
	fmt.Println(strings.ToTitle("хлеб"))

	// ToTitleSpecial
	fmt.Println(strings.ToTitleSpecial(unicode.TurkishCase, "dünyanın ilk borsa yapısı Aizonai kabul edilir"))

	// ToValidUTF8
	fmt.Printf("%s\n", strings.ToValidUTF8("abc", "\uFFFD"))
	fmt.Printf("%s\n", strings.ToValidUTF8("a\xffb\xC0\xAFc\xff", ""))
	fmt.Printf("%s\n", strings.ToValidUTF8("\xed\xa0\x80", "abc"))

}

// 清除 Unicode 字符
func TestTrim(t *testing.T) {
	fmt.Print(strings.Trim("¡¡¡Hello, Gophers!!!", "!¡"))
}
